use random;

/* Generate instance data */
function input() {
    nbItems = 10;
    nbBins = 2;
    nbScenarios = 3;
    rngSeed = 42;

    // Pick random parameters for each item distribution
    rng = random.create(rngSeed);
    itemsMin[i in 0...nbItems] = rng.next(10, 101);
    itemsMax[i in 0...nbItems] = itemsMin[i] + rng.next(0, 51);

    // Sample the distributions to generate the scenarios
    scenarioItemWeights[i in 0...nbScenarios][j in 0...nbItems] = 
            rng.next(itemsMin[j], itemsMax[j] + 1);
}

/* Declare the optimization model */
function model() {
    // Set decisions: bins[k] represents the items in bin k
    bins[k in 0...nbBins] <- set(nbItems);

    // Each item must be in one bin and one bin only
    constraint partition[k in 0...nbBins](bins[k]);

    // Compute max weight for each scenario
    scenarioMaxWeight[m in 0...nbScenarios] <- max[k in 0...nbBins](
            sum(bins[k], i => scenarioItemWeights[m][i]));

    // Compute the 9th decile of scenario max weights
    stochasticMaxWeight <- sort(scenarioMaxWeight)[ceil(0.9 * (nbScenarios - 1))];

    minimize stochasticMaxWeight;
}

// Parametrize the solver
function param() {
    if (lsTimeLimit == nil) lsTimeLimit = 2;
}

/* Write the solution */
function output() {
    println();
    println("Scenario item weights:");
    for [i in 0...nbScenarios] {
        print(i + ": [");
        for [j in 0...nbItems]
            print(scenarioItemWeights[i][j] + (j == nbItems - 1 ? "" : ", "));
        println("]");
    }

    println();
    println("Bins:");
    for [k in 0...nbBins]
        println(k + ": " + bins[k].value);
}
